@@ -295,6 +295,10 @@ def test_install_python_bin():
 
         for pth in required_executables:
             assert os.path.exists(os.path.join(bin_dir, pth)), "%s should exist in bin_dir" % pth
+        if not virtualenv.IS_JYTHON:
+            assert os.path.exists(inc_dir)
+        root_inc_dir = os.path.join(home_dir, "include")
+        assert not os.path.islink(root_inc_dir)
     finally:
         shutil.rmtree(tmp_virtualenv)
 
@@ -544,6 +548,33 @@ def test_create_environment_with_exec_prefix_pointing_to_prefix(tmpdir):
         os.environ[path_key] = old_path
 
 
+@pytest.mark.skipif(not hasattr(os, "symlink"), reason="requires working symlink implementation")
+def test_relative_symlink_relative_target(tmp_path):
+    """Test that copyfile works correctly when the source is a symlink with a
+    relative target, and a symlink to a symlink."""
+
+    # Set up src/link2 -> ../src/link1 -> file.
+    # We will copy to a different directory, so misinterpreting either symlink
+    # will be detected.
+    src_dir = tmp_path / "src"
+    src_dir.mkdir()
+    with open(str(src_dir / "file"), "w") as f:
+        f.write("contents")
+    os.symlink("file", str(src_dir / "link1"))
+    os.symlink(str(Path("..") / "src" / "link1"), str(src_dir / "link2"))
+
+    # Check that copyfile works on link2.
+    # This may produce a symlink or a regular file depending on the platform --
+    # which doesn't matter as long as it has the right contents.
+    copy_path = tmp_path / "copy"
+    virtualenv.copyfile(str(src_dir / "link2"), str(copy_path))
+    with open(str(copy_path), "r") as f:
+        assert f.read() == "contents"
+
+    shutil.rmtree(str(src_dir))
+    os.remove(str(copy_path))
+
+
 @pytest.mark.skipif(not hasattr(sys, "real_prefix"), reason="requires running from inside virtualenv")
 def test_create_environment_from_virtual_environment(tmpdir):
     """Create virtual environment using Python from another virtual environment
@@ -552,3 +583,15 @@ def test_create_environment_from_virtual_environment(tmpdir):
     home_dir, lib_dir, inc_dir, bin_dir = virtualenv.path_locations(venvdir)
     virtualenv.create_environment(venvdir)
     assert not os.path.islink(os.path.join(lib_dir, "distutils"))
+
+
+def test_create_environment_with_old_pip(tmpdir):
+    old = Path(__file__).parent / "old-wheels"
+    old_pip = old / "pip-9.0.1-py2.py3-none-any.whl"
+    old_setuptools = old / "setuptools-30.4.0-py2.py3-none-any.whl"
+    support_dir = str(tmpdir / "virtualenv_support")
+    os.makedirs(support_dir)
+    for old_dep in [old_pip, old_setuptools]:
+        shutil.copy(str(old_dep), support_dir)
+    venvdir = str(tmpdir / "venv")
+    virtualenv.create_environment(venvdir, search_dirs=[support_dir], no_wheel=True)
